@@ -37,14 +37,11 @@
 #include <linux/telephony.h>
 /* Still use some IXJ specific stuff */
 #include <linux/version.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-# include <linux/compiler.h>
-#endif
 #include <linux/ixjuser.h>
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 41411 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 48087 $")
 
 #include "asterisk/lock.h"
 #include "asterisk/channel.h"
