@@ -37,9 +37,11 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 44955 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 56729 $")
 
+#define AST_API_MODULE
 #include "asterisk/lock.h"
+
 #include "asterisk/io.h"
 #include "asterisk/logger.h"
 #include "asterisk/md5.h"
@@ -195,6 +197,7 @@ struct hostent *ast_gethostbyname(const char *host, struct ast_hostent *hp)
 		if (dots != 3)
 			return NULL;
 		memset(hp, 0, sizeof(struct ast_hostent));
+		hp->hp.h_addrtype = AF_INET;
 		hp->hp.h_addr_list = (void *) hp->buf;
 		hp->hp.h_addr = hp->buf + sizeof(void *);
 		if (inet_pton(AF_INET, host, hp->hp.h_addr) > 0)
@@ -519,7 +522,7 @@ char *ast_strip_quoted(char *s, const char *beg_quotes, const char *end_quotes)
 	char *q;
 
 	s = ast_strip(s);
-	if ((q = strchr(beg_quotes, *s))) {
+	if ((q = strchr(beg_quotes, *s)) && *q != '\0') {
 		e = s + strlen(s) - 1;
 		if (*e == *(end_quotes + (q - beg_quotes))) {
 			s++;
@@ -906,3 +909,14 @@ void ast_enable_packet_fragmentation(int sock)
 #endif
 }
 
+AST_MUTEX_DEFINE_STATIC(fetchadd_m); /* used for all fetc&add ops */
+
+int ast_atomic_fetchadd_int_slow(volatile int *p, int v)
+{
+        int ret;
+        ast_mutex_lock(&fetchadd_m);
+        ret = *p;
+        *p += v;
+        ast_mutex_unlock(&fetchadd_m);
+        return ret;
+}
